/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import javax.swing.ListModel;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.panels.request.AbstractWsdlRequestDesktopPanel;
import com.eviware.soapui.impl.wsdl.panels.request.components.SoapMessageXmlEditor;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.assertions.Assertable.AssertionStatus;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.actions.AddAssertionAction;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.iface.Request.SubmitException;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.monitor.support.TestMonitorListenerAdapter;
import com.eviware.soapui.support.ListDataChangeListener;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JComponentInspector;
import com.eviware.soapui.support.components.JInspectorPanel;
import com.eviware.soapui.support.log.JLogList;

/**
 * DesktopPanel for WsdlTestRequest. Essentially a copy of WsdlRequestDesktopPanel with assertions.
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequestDesktopPanel extends AbstractWsdlRequestDesktopPanel<WsdlTestRequestStep, WsdlTestRequest> implements PropertyChangeListener
{
   private JLogList logArea;
   private InternalTestMonitorListener testMonitorListener = new InternalTestMonitorListener();
	private JButton addAssertionButton;
	protected boolean updatingRequest;
	private WsdlTestRequestDesktopPanel.InternalSubmitListener submitListener;
	private AssertionsPanel assertionsPanel;
	private JInspectorPanel inspectorPanel;
	private JComponentInspector assertionInspector;
	private JComponentInspector logInspector;
	
	public WsdlTestRequestDesktopPanel( WsdlTestRequestStep requestStep )
   {
      super( requestStep );
      
      init( requestStep.getTestRequest() );
      
      SoapUI.getTestMonitor().addTestMonitorListener( testMonitorListener );
      setEnabled( !SoapUI.getTestMonitor().hasRunningTest( requestStep.getTestCase() ) );
      
      requestStep.getTestRequest().addPropertyChangeListener( WsdlTestRequest.STATUS_PROPERTY, this );
   }

   protected JComponent buildLogPanel()
   {
      logArea = new JLogList("Request Log");
      
      logArea.getLogList().getModel().addListDataListener( new ListDataChangeListener() {

			public void dataChanged( ListModel model )
			{
				logInspector.setTitle( "Request Log (" + model.getSize() + ")" );
			}}
				);
      
      return logArea;
   }
   
   protected AssertionsPanel buildAssertionsPanel()
   {
   	return new AssertionsPanel( getRequest() )
   	{
			protected void selectError(AssertionError error)
			{
				SoapMessageXmlEditor editor = getResponseEditor();
				editor.requestFocus();
			}
   	};
   }
   
	public void setContent(JComponent content)
	{
		inspectorPanel.setContentComponent( content );
	}

	public void removeContent(JComponent content)
	{
		inspectorPanel.setContentComponent( null );
	}
	
	protected String getHelpUrl()
	{
		return HelpUrls.TESTREQUESTEDITOR_HELP_URL;
	}

	protected JComponent buildContent()
   {
   	JComponent component = super.buildContent();
   	
   	inspectorPanel = new JInspectorPanel( component );
   	assertionsPanel = buildAssertionsPanel();
		assertionInspector = inspectorPanel.addInspector( new JComponentInspector( assertionsPanel, "Assertions", 
				   				"Assertions for this Test Request", true) );
		
   	logInspector = new JComponentInspector( buildLogPanel(), "Request Log (0)", "Log of requests", true);
		inspectorPanel.addInspector( logInspector );
   	inspectorPanel.setDefaultDividerLocation( 0.6F );
   	inspectorPanel.setCurrentInspector( "Assertions" );
   	
   	updateStatusIcon();
   	
   	return inspectorPanel;
   }

	private void updateStatusIcon()
	{
		AssertionStatus status = getModelItem().getTestRequest().getAssertionStatus();
   	switch( status )
   	{
   		case FAILED : 
			{
				assertionInspector.setIcon( UISupport.createImageIcon( "/failed_assertion.gif" ));
				inspectorPanel.activate( assertionInspector ); 
				break;
			}
   		case UNKNOWN : 
			{
				assertionInspector.setIcon( UISupport.createImageIcon( "/unknown_assertion.gif" )); 
				break;
			}
   		case VALID : 
			{
				assertionInspector.setIcon( UISupport.createImageIcon( "/valid_assertion.gif" ));
				inspectorPanel.deactivate(); 
				break;
			}
   	}
	}
   
   protected JComponent buildToolbar()
	{
   	addAssertionButton = createActionButton(new AddAssertionAction(getRequest()), true );
		return super.buildToolbar();
	}

	protected void insertButtons(JToolBar toolbar)
	{
		toolbar.add( addAssertionButton );
	}

	public void setEnabled( boolean enabled )
   {
		if( enabled == true )
			enabled = !SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem().getTestCase() );
		
		super.setEnabled( enabled );
      addAssertionButton.setEnabled( enabled );
      assertionsPanel.setEnabled( enabled );
      
      if( SoapUI.getTestMonitor().hasRunningLoadTest( getRequest().getTestCase() ))
      {
      	getRequest().removeSubmitListener( submitListener );
      }
      else
      {
      	getRequest().addSubmitListener( submitListener );
      }
   }
   
   protected Submit doSubmit() throws SubmitException
	{
		return getRequest().submit( new WsdlTestRunContext(getModelItem()), true );
	}

   protected InternalSubmitListener createSubmitListener()
	{
		submitListener = new InternalSubmitListener();
		return submitListener;
	}
   
	private class InternalSubmitListener extends AbstractWsdlRequestDesktopPanel.InternalSubmitListener
   {
      private long startTime;
      private SimpleDateFormat sdf;

      private InternalSubmitListener()
      {
      	super();
         sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      }

      public boolean beforeSubmit(Submit submit, SubmitContext context)
      {
      	boolean result = super.beforeSubmit( submit, context );
         startTime = System.currentTimeMillis();
         return result;
      }

      protected void logMessages(String message, String infoMessage)
		{
         super.logMessages( message, infoMessage );
         logArea.addLine( sdf.format( new Date( startTime )) + " - " + message );
      }

		@Override
		public void afterSubmit( Submit submit, SubmitContext context )
		{
			super.afterSubmit( submit, context );
			updateStatusIcon();
		}
   }
   
	public boolean onClose( boolean canCancel )
	{
		if( super.onClose( canCancel ))
		{
			assertionsPanel.release();
			SoapUI.getTestMonitor().removeTestMonitorListener( testMonitorListener );
			logArea.release();
			getModelItem().getTestRequest().removePropertyChangeListener( WsdlTestRequest.STATUS_PROPERTY, this );
			return true;
		}
		
		
      return false;
	}
	
	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == getRequest() || modelItem == getModelItem() || modelItem == getRequest().getOperation() ||
		    modelItem == getRequest().getOperation().getInterface() || 
		    modelItem == getRequest().getOperation().getInterface().getProject() ||
		    modelItem == getModelItem().getTestCase() || modelItem == getModelItem().getTestCase().getTestSuite();
	}
	
	private class InternalTestMonitorListener extends TestMonitorListenerAdapter
	{
		public void loadTestFinished(LoadTestRunner runner)
		{
			setEnabled( !SoapUI.getTestMonitor().hasRunningTest( getModelItem().getTestCase() ) );
		}

		public void loadTestStarted(LoadTestRunner runner)
		{
			if( runner.getLoadTest().getTestCase() == getModelItem().getTestCase() )
				setEnabled( false );
		}

		public void testCaseFinished(TestRunner runner)
		{
			setEnabled(	!SoapUI.getTestMonitor().hasRunningTest( getModelItem().getTestCase() ) );
		}

		public void testCaseStarted(TestRunner runner)
		{
			if( runner.getTestCase() == getModelItem().getTestCase())
				setEnabled( false );
		}
	}

	public void propertyChange( PropertyChangeEvent evt )
	{
		updateStatusIcon();
	}
}
